/* SPDX-License-Identifier: BSD-3-Clause
 * Copyright (c) 2022 NVIDIA Corporation & Affiliates
 */

#ifndef MLX5DR_DEV_H_
#define MLX5DR_DEV_H_

struct mlx5dr_dev_context;
struct mlx5dr_dev_matcher;
struct mlx5dr_dev_action;

struct mlx5dr_dev_action_ctv4 {
	struct mlx5dr_dev_action *ctr;
	struct mlx5dr_dev_action *single_modify;
	struct mlx5dr_dev_action *dest_table;
	uint32_t ctr_offset;
	uint32_t modify_value;
};

struct mlx5dr_dev_rule_ctv4 {
	__be16 src_port;
	__be16 dst_port;
	__be32 src_addr;
	__be32 dst_addr;
	__be32 metadata;
	uint8_t protocol;
};

struct mlx5dr_dev_rule {
	union {
		struct mlx5dr_dev_rule_ctv4 ctv4;
	};
	/* mlx5dr internal */
	uint32_t final_rtc_p0;
	uint32_t final_rtc_p1;
	uint8_t pending_wqes;
	uint8_t status;
};

struct mlx5dr_dev_rule_attr {
	uint16_t queue_id;
	void *user_data;
	/* Valid if matcher optimize_using_rule_idx is set */
	uint32_t rule_idx;
	uint32_t burst:1;
};

enum mlx5dr_dev_send_op_status {
	MLX5DR_DEV_SEND_OP_SUCCESS,
	MLX5DR_DEV_SEND_OP_ERROR,
};

struct mlx5dr_dev_send_op_result {
	enum mlx5dr_dev_send_op_status status;
	void *user_data;
};

/* Bind thread context to thread. This initialization should be called once per
 * new thread to allow rule create/destroy/poll operations over the provided
 * queue id. For DPA flexio_dev_outbox_config must be called prior to this call.
 * NOTE: This function should only be called from the device.
 *
 * @param[in] dev_ctx - Device context.
 * @return void
 */
void mlx5dr_dev_send_bind_to_thread(struct mlx5dr_dev_context *dev_ctx,
				    uint8_t queue_id);

/* Create CT ipv4 rule.
 * NOTE: This function should only be called from the device.
 *
 * @param[in] dev_matcher - FDB DPA device matcher supporting ipv4 CT.
 * @param[in] attr - Rule creation attributes.
 * @param[in, out] dev_rule - Rule handle set with the 5tuple matching fields.
 * @param[in] dev_actions - CT DPA device actions.
 * @return 0 on success non zero otherwise.
 */
int mlx5dr_dev_rule_ctv4_create(struct mlx5dr_dev_matcher *dev_matcher,
				struct mlx5dr_dev_rule_attr *attr,
				struct mlx5dr_dev_rule *dev_rule,
				struct mlx5dr_dev_action_ctv4 *dev_actions);

/* Destroy CT ipv4 rule.
 * NOTE: This function should only be called from the device.
 *
 * @param[in] dev_matcher - Device matcher.
 * @param[in] attr - Rule creation attributes.
 * @param[in] dev_rule- Rule handle to destroy.
 * @return 0 on success non zero otherwise.
 */
int mlx5dr_dev_rule_ctv4_destroy(struct mlx5dr_dev_matcher *dev_matcher,
				 struct mlx5dr_dev_rule_attr *attr,
				 struct mlx5dr_dev_rule *dev_rule);

/* Poll queue for rule creation and deletions completions.
 * NOTE: This function should only be called from the device.
 *
 * @param[in] dev_ctx - Device context.
 * @param[in] queue_id - The id of the queue to poll.
 * @param[in, out] res - Completion array.
 * @param[in] res_nb - Maximum number of results to return.
 * @return negative number on failure, the number of completions otherwise.
 */
int mlx5dr_dev_send_queue_poll(struct mlx5dr_dev_context *dev_ctx,
			       uint16_t queue_id,
			       struct mlx5dr_dev_send_op_result res[],
			       uint32_t res_nb);

/* Order queue to process queue rule creation and deletion
 * NOTE: This function should only be called from the device.
 *
 * @param[in] dev_ctx - Device context.
 * @param[in] queue_id - The id of the queue to poll.
 * @return 0 on success non zero otherwise.
 */
int mlx5dr_dev_send_queue_drain(struct mlx5dr_dev_context *dev_ctx,
				uint16_t queue_id);

#endif
